@@ -13,7 +13,7 @@
 ##############################################################################
 """Component Architecture Tests
 
-$Id: tests.py 105528 2009-11-08 11:05:28Z kobold $
+$Id: tests.py 108364 2010-01-21 17:31:43Z janwijbrand $
 """
 import persistent
 import re
@@ -1706,6 +1706,10 @@ def test_suite():
                              setUp=setUp, tearDown=tearDown),
         doctest.DocFileSuite('zcml.txt', checker=checker,
                              setUp=setUp, tearDown=tearDown),
+        doctest.DocFileSuite('testlayer.txt',
+                             optionflags=(doctest.ELLIPSIS +
+                                          doctest.NORMALIZE_WHITESPACE +
+                                          doctest.REPORT_NDIFF)),
         zcml_conditional,
         hooks_conditional,
         unittest.makeSuite(StandaloneTests),
